"use strict";
// ------------------------------------------------------------------------------
//  Copyright (c) 2017-present, RobotlegsJS. All Rights Reserved.
//
//  NOTICE: You are permitted to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
// ------------------------------------------------------------------------------
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@robotlegsjs/core");
var AsyncCommand_1 = require("./AsyncCommand");
var SubCommandMapping_1 = require("./SubCommandMapping");
var SubCommandMappingList_1 = require("./SubCommandMappingList");
var AbstractMacro = /** @class */ (function (_super) {
    tslib_1.__extends(AbstractMacro, _super);
    function AbstractMacro(context, injector) {
        var _this = _super.call(this, context) || this;
        _this._injector = injector.createChild();
        _this._mappings = new SubCommandMappingList_1.SubCommandMappingList();
        _this.prepare();
        return _this;
    }
    AbstractMacro.prototype.add = function (commandClass) {
        var mapping = new SubCommandMapping_1.SubCommandMapping(commandClass);
        this._mappings.addMapping(mapping);
        return mapping;
    };
    AbstractMacro.prototype.remove = function (commandClass) {
        this._mappings.removeMappingsFor(commandClass);
    };
    AbstractMacro.prototype.executeCommand = function (mapping) {
        var command;
        var commandClass = mapping.commandClass;
        var payloads = mapping.payloads;
        var hasPayloads = payloads.length > 0;
        if (hasPayloads) {
            this.mapPayloads(payloads);
        }
        if (mapping.guards.length === 0 || core_1.guardsApprove(mapping.guards, this._injector)) {
            command = mapping.getOrCreateCommandInstance(this._injector);
            if (mapping.hooks.length > 0) {
                this._injector.bind(commandClass).toConstantValue(command);
                core_1.applyHooks(mapping.hooks, this._injector);
                this._injector.unbind(commandClass);
            }
        }
        if (hasPayloads) {
            this.unmapPayloads(payloads);
        }
        if (command) {
            var isAsync = command.constructor.prototype.registerCompleteCallback !== undefined;
            if (isAsync) {
                command.registerCompleteCallback(this.commandCompleteHandler.bind(this));
            }
            command.execute();
            if (!isAsync) {
                this.commandCompleteHandler(true);
            }
        }
        else {
            this.commandCompleteHandler(true);
        }
    };
    AbstractMacro.prototype.mapPayloads = function (payloads) {
        this._payloadsModule = new core_1.ContainerModule(function (bind, unbind) {
            payloads.forEach(function (payload) {
                if (payload.name.length > 0) {
                    bind(payload.type)
                        .toConstantValue(payload.data)
                        .whenTargetNamed(payload.name);
                }
                else {
                    bind(payload.type).toConstantValue(payload.data);
                }
            });
        });
        this._injector.load(this._payloadsModule);
    };
    AbstractMacro.prototype.unmapPayloads = function (payloads) {
        this._injector.unload(this._payloadsModule);
    };
    AbstractMacro = tslib_1.__decorate([
        core_1.injectable(),
        tslib_1.__param(0, core_1.inject(core_1.IContext)), tslib_1.__param(1, core_1.inject(core_1.IInjector)),
        tslib_1.__metadata("design:paramtypes", [Object, Object])
    ], AbstractMacro);
    return AbstractMacro;
}(AsyncCommand_1.AsyncCommand));
exports.AbstractMacro = AbstractMacro;
//# sourceMappingURL=AbstractMacro.js.map